require("pgcommands")

function Definitions()
	DebugMessage("%s -- Defining custom freestore movement perceptions", tostring(Script))

	-- Table which maps heroes to perceptions for systems they like to hang out on when not in active use
	-- The boolean is for whether or not the hero prefers to stay in space, if he has a choice
	-- Generally, this is to find the system where their abilities provide the best defensive or infrastructure bonuses
	CustomUnitPlacement = {
		BAST_TEAM = {nil, true},
		BOBA_FETT_TEAM = {nil, true},
		CRONAL_TEAM = {nil, true},
		DARTH_TEAM = {nil, false},
		FIRMUS_PIETT_TEAM = {nil, true},
		MAAREK_STELE_TEAM = {nil, true},
		MARA_JADE_TEAM = {nil, false},
		GENERAL_VEERS_TEAM = {nil, false},
		EMPEROR_PALPATINE_TEAM = {nil, false},
		ROM_MOHC_TEAM = {nil, true},
		SATE_PESTAGE_TEAM = {nil, false},
		SOONTIR_FEL_TEAM = {nil, true},
		THRAWN_TEAM = {"Is_Research_Planet", true},
		GRAND_MOFF_TARKIN_TEAM = {nil, true},

		COMMANDER_AKBAR_TEAM = {"Is_Research_Planet", true},
		BAIL_ORGANA_TEAM = {nil, false},
		DROIDS_TEAM = {nil, false},
		GARM_BEL_IBLIS_TEAM = {nil, false},
		HAN_SOLO_TEAM = {nil, false},
		GENERAL_DODONNA_TEAM = {"Is_Research_Planet", true},
		KATARN_TEAM = {nil, false},
		LEIA_ORGANA_TEAM = {nil, false},
		LUKE_SKYWALKER_JEDI_TEAM = {nil, false},
		MON_MOTHMA_TEAM = {nil, false},
		OBI_WAN_TEAM = {nil, false},
		WEDGE_ANTILLES_TEAM = {nil, true},
		YODA_TEAM = {nil, false}
	}

end

function Find_Custom_Target(object)
	object_type = object.Get_Type()
	object_type_name = object_type.Get_Name()

	unit_entry = CustomUnitPlacement[object_type_name]

	if unit_entry then
		perception = unit_entry[1]
		prefers_space = unit_entry[2]
		if perception then
			target = FindTarget.Reachable_Target(PlayerObject, perception, "Friendly", "No_Threat", 1.0, object)
			if TestValid(target) then
				return target
			end
		end
		
		if prefers_space then
			return Find_Space_Unit_Target(object)
		else
			return Find_Ground_Unit_Target(object)
		end
	else
		DebugMessage("%s -- Error: Type %s not found in CustomUnitPlacement table.", tostring(Script), object_type_name)
	end
end
